/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.MissingServerPropertyException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.topology.generated.Topology;

/**
 * <p>
 * The interface of the Configuration service that handles the configuration of the local PEtALS container.
 * </p>
 * <p>
 * During the execution of methods {@link #loadConfiguration(Properties, Topology)} or {@link #updateTopology(Topology, boolean)}
 * , the other methods are blocked until the end of the configuration and topology loading.
 * </p>
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @author Charles Casadei - EBM WebSourcing
 * @since Petals 2.0
 */
public interface ConfigurationService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.ConfigurationService";

    /**
     * Name of the server interface of the Fractal component
     * {@value FractalHelper#CONFIGURATION_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";
    
    /**
     * Get the local container configuration.
     * 
     */
    ContainerConfiguration getContainerConfiguration();

    /**
     * <p>
     * Get the local server properties (ie. content of the file '
     * <code>server.properties</code>' or equivalent), <b>including properties
     * not configured</b> (ie. set to a <code>null</code> value).
     * </p>
     * <p>
     * Note: local server properties are provided as a {@link Map} instead of
     * {@link Properties} to be able to put <code>null</code> values.
     * </p>
     * 
     * 
     */
    Map<String, String> getAllServerProperties();
    
    /**
     * Get the local server properties (ie. content of the file ' <code>server.properties</code>' or equivalent),
     * <b>including only properties configured</b> (ie. set to a non <code>null</code> value).
     * 
     * @return The local server properties. Cannot be <code>null</code>.
     */
    public Properties getServerProperties();

    /**
     * @return The container configuration of remote containers. The local container configuration is returned by
     *         {@link #getContainerConfiguration()}
     */
    public Set<ContainerConfiguration> getContainersConfiguration();

    /**
     * Get the domain configuration the local container belongs to.
     * 
     * @return
     */
    DomainConfiguration getDomainConfiguration();

    /**
     * @return The topology read from the local topology definition (ie. configured or persisted topology.xml).
     */
    public Topology getLocalTopology();

    /**
     * Get the container configuration.
     * 
     * @param containerName
     *            the name of the container to get configuration from. If null, returns the local container
     *            configuration.
     * @return The container configuration of the container named <code>containerName</code>, or <code>null</code> if
     *         not found.
     */
    ContainerConfiguration getContainerConfiguration(final String containerName);

    /**
     * <p>
     * Create the container configuration based on the local topology and server configuration properties. Basic
     * consistency is controlled between the 2 configurations.
     * </p>
     * <p>
     * Required directories are created when loading the configuration: working area directory.
     * </p>
     * <p>
     * <b>Note 1</b>: If present into the server configuration properties, the parameter <code>petals.topology.url</code>
     * is ignored, the topology is provided through the parameter '<code>topology</code>'.
     * <p>
     * <b>Note 2: MUST be invoked just after having starting the Configuration service and before any other component
     * starts.</b>
     * </p>
     * <p>
     * <b>Note 3</b>: If a topology definition exists in the working area, the topology definition '
     * <code>topology</code>' is replaced by this persisted topology definition. The persisted topology aims to
     * persisted topology changes that can not be stored in the topology definition reference because we are not able to
     * manage all URL scheme.
     * </p>
     * 
     * @param serverLocalProperties
     *            The local properties of the current container (ie: the content of 'server.properties' or equivalent).
     *            Can be <code>null</code>.
     * @param topology
     *            The topology configuration. Can not be <code>null</code>.
     * @throws MissingServerPropertyException
     *             A server property is missing.
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    public void loadConfiguration(final Properties serverLocalProperties, final Topology topology)
            throws ConfigurationException;

    /**
     * <p>
     * Update the container configuration according to a topology definition. Basic consistency is controlled between
     * the existing topology and the new one.
     * </p>
     * 
     * @param topology
     *            The new topology to apply. Can not be <code>null</code>.
     * @param validate
     *            if validations should be done
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    public void updateTopology(final Topology topology, final boolean validate)
            throws ConfigurationException;

    /**
     * <p>
     * Reconfigure the current container and domain with the given domain configuration.
     * </p>
     * <p>
     * No checks is done before to reconfigure
     * </p>
     * 
     * @param topology
     */
    public void reconfigureDomain(final Topology topology) throws ConfigurationException;
    
    /**
     * Validate the given security topology pass-phrase against the one configured at the current container.
     * 
     * @param securityPassPhrase
     *            The security topology pass-phrase to check.
     * @return <code>true</code> if the security topology pass-phrase is not null, non-empty and validated,
     *         <code>false</code> otherwise.
     */
    public boolean isSecurityTopologyPassphraseOk(final String securityPassPhrase);

    /**
     * Register a listener on events about remote container configuration of the domain of the current container.
     * 
     * @param listener
     *            The listener to register
     */
    public void registerContainerConfigurationListener(final ContainerConfigurationListener listener);

    /**
     * Deregister a listener on events about remote container configuration of the domain of the current container.
     * 
     * @param listener
     *            The listener to deregister
     */
    public void deregisterContainerConfigurationListener(final ContainerConfigurationListener listener);

}
