/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.messaging.endpoint;

import java.util.Collection;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.clientserverapi.jbi.messaging.endpoint.exception.EndpointDirectoryException;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.jbi.messaging.endpoint.exception.EndpointAlreadyExistsException;
import org.ow2.petals.microkernel.api.jbi.messaging.endpoint.exception.EndpointDoesNotExistsException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * This class is used to manage everything that deals with endpoint in the distributed environment.
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 */
public interface EndpointDirectoryService extends EndpointDirectoryMBean {

    public static final String COMPONENT_LOGGER_NAME = "Petals.JBI-Messaging.EndpointDirectoryService";

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#ENDPOINT_DIRECTORY_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * The name of the client interface of the component {@value FractalHelper#ENDPOINT_DIRECTORY_COMPONENT} for the
     * shared area service.
     */
    public final static String SHARED_AREA_FRACTAL_ITF_NAME = "sharedArea";

    /**
     * The name of the client interface of the component {@value FractalHelper#ENDPOINT_DIRECTORY_COMPONENT} for the
     * container service.
     */
    public final static String CONTAINER_FRACTAL_ITF_NAME = "container";

    /**
     * The name of the client interface of the component {@value FractalHelper#ENDPOINT_DIRECTORY_COMPONENT} for the
     * configuration service.
     */
    public final static String CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    public Collection<PetalsServiceEndpoint> getInternalEndpoints() throws EndpointDirectoryException;

    public Collection<PetalsServiceEndpoint> getExternalEndpoints() throws EndpointDirectoryException;

    /**
     * <p>
     * Register an endpoint in the endpoint directory.
     * </p>
     * 
     * <b>Note</b>: If the endpoint already exists and <code>canOverwrite</code> is set to <code>true</code>, it is
     * overridden. Otherwise the error {@link EndpointAlreadyExistsException} will occur. The overwriting is required by
     * the recovering service.
     * 
     * @param serviceName
     *            Endpoint service name.
     * @param endpointName
     *            Endpoint name
     * @param interfaces
     *            Endpoint interfaces
     * @param description
     *            Endpoint WSDL
     * @param componentName
     *            the name of the component activating this endpoint
     * @return A {@link PetalsServiceEndpoint} associated to the registered endpoint.
     * @throws EndpointAlreadyExistsException
     *             The endpoint already exists.
     * @throws EndpointDirectoryException
     *             An error occurs activating the endpoint
     */
    public PetalsServiceEndpoint activateEndpoint(final QName serviceName, final String endpointName,
            final QName[] interfaces, final Document description, final String componentName)
            throws EndpointDirectoryException;

    /**
     * Deactivates the local given endpoint. Does not care about remotes ones !
     * 
     * @param endpointName
     *            Endpoint name. Not null.
     * @param serviceName
     *            Endpoint service name. Not null.
     * @param componentName
     *            the name of the component activating this endpoint. Not null.
     * @throws EndpointDoesNotExistsException
     *             The endpoint is not activated.
     * @throws EndpointDirectoryException
     *             if the endpoint cannot be deactivated
     */
    public void deactivateEndpoint(final String endpointName, final QName serviceName, final String componentName)
            throws EndpointDirectoryException;

    /**
     * Register this given external endpoint
     * 
     * <b>Note</b>: If the endpoint already exists and <code>canOverwrite</code> is set to <code>true</code>, it is
     * overridden. Otherwise the error {@link EndpointAlreadyExistsException} will occur. The overwriting is required by
     * the recovering service.
     * 
     * @param serviceEndpoint
     *            the external endpoint to be registered; must be non null
     * @param componentName
     *            the name of the component activating this endpoint; must be non null
     * @throws EndpointAlreadyExistsException
     *             The endpoint already exists.
     * @throws EndpointDirectoryException
     *             if the external endpoint cannot be registered
     */
    public void registerExternalEndpoint(final ServiceEndpoint serviceEndpoint, final String componentName)
            throws EndpointDirectoryException;

    /**
     * Deregister external endpoint
     * 
     * @param externalEndpoint
     *            external to be deregistered; must be non null
     * @param componentName
     *            the name of the component activating this endpoint; must be non null
     * @throws EndpointDoesNotExistsException
     *             The endpoint is not registered.
     * @throws EndpointDirectoryException
     *             if the endpoint cannot be deregistered
     */
    public void deregisterExternalEndpoint(final ServiceEndpoint externalEndpoint, final String componentName)
            throws EndpointDirectoryException;

    /**
     * Register a connection
     * 
     * TODO service connection they must be restricted to a given SA, see PETALSESBCONT-391
     * 
     * @param consInterface
     *            consumer interface name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws EndpointDirectoryException
     *             if a problem occurs in the registry
     */
    public void registerConnection(final QName consInterface, final QName provService, final String provEndpoint)
            throws EndpointDirectoryException;

    /**
     * Register a connection
     * 
     * TODO service connection they must be restricted to a given SA, see PETALSESBCONT-391
     * 
     * @param consService
     *            consumer service name; must be non null
     * @param consEndpoint
     *            consumer endpoint name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws EndpointDirectoryException
     *             if a problem occurs in the registry
     */
    public void registerConnection(final QName consService, final String consEndpoint, final QName provService,
            final String provEndpoint) throws EndpointDirectoryException;

    /**
     * Delete a connection
     * 
     * TODO service connection they must be restricted to a given SA, see PETALSESBCONT-391
     * 
     * @param consInterface
     *            consumer interface name; must be non null
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws EndpointDirectoryException
     *             if the connection can not be deleted
     */
    public void deregisterConnection(final QName consInterface, final QName provService, final String provEndpoint)
            throws EndpointDirectoryException;

    /**
     * Delete a connection
     * 
     * TODO service connection they must be restricted to a given SA, see PETALSESBCONT-391
     * 
     * @param consService
     *            consumer service name; must be non null
     * @param consEndpoint
     *            consumer endpoint name; must be non null
     * 
     * @param provService
     *            provider service name; must be non null
     * @param provEndpoint
     *            provider endpoint name; must be non null
     * @throws EndpointDirectoryException
     *             if the connection can not be deleted
     */
    public void deregisterConnection(final QName consService, final String consEndpoint, final QName provService,
            final String provEndpoint) throws EndpointDirectoryException;

    /**
     * Queries the registry for active endpoints that implements the given interface, according to the link type.
     * 
     * @param interfaceName
     *            qualified interface/portType that is implemented by the endpoints; must be non-null
     * @return a collection of available endpoints for the specified interface name; must be non null, may be empty
     */
    public Collection<PetalsServiceEndpoint> getInternalEndpointsForInterface(QName interfaceName)
            throws EndpointDirectoryException;

    /**
     * Queries the registry for active endpoints belonging to the given service, according to the link type.
     * 
     * @param serviceName
     *            qualified name of the service that the endpoints are part of; must be non null
     * @return a collection of available endpoints for the specified service name; must be non null, may be empty
     */
    public Collection<PetalsServiceEndpoint> getInternalEndpointsForService(QName serviceName)
            throws EndpointDirectoryException;

    /**
     * Queries the registry for external endpoints that implements the given interface name
     * 
     * @param interfaceName
     *            qualified name of interface implemented by the endpoints; must be non null
     * @return a collection of available endpoints for the specified interface name; must be non null, may be empty
     */
    public Collection<PetalsServiceEndpoint> getExternalEndpointsForInterface(QName interfaceName)
            throws EndpointDirectoryException;

    /**
     * Queries the registry for external endpoints that are part of the given service name
     * 
     * @param serviceName
     *            qualified name of service that contains endpoints; must be non null
     * @return a collection of available endpoints for the specified service name; must be non null, may be empy
     */
    public Collection<PetalsServiceEndpoint> getExternalEndpointsForService(QName serviceName)
            throws EndpointDirectoryException;

    /**
     * Get the service endpoint for the named activated endpoint, if any.
     * 
     * @param service
     *            qualified name of the endpoint's service; must be non null
     * @param name
     *            name of the endpoint; must be non null
     * @return the named endpoint, or null if it is not activated
     */
    public PetalsServiceEndpoint getInternalEndpoint(QName service, String name) throws EndpointDirectoryException;
    
    /**
     * Get the service endpoint for the named registered external endpoint, if any.
     * 
     * @param service
     *            qualified name of the external endpoint's service; must be non null
     * @param name
     *            name of the external endpoint; must be non null
     * @return the named external endpoint, or null if it is not registered
     */
    public PetalsServiceEndpoint getExternalEndpoint(QName service, String name) throws EndpointDirectoryException;

    /**
     * Get all endpoints activated, including external endpoints.
     * 
     * @return All endpoints activated
     * @throws EndpointDirectoryException
     */
    public Collection<PetalsServiceEndpoint> getEndpoints() throws EndpointDirectoryException;

    /**
     * Returns the ServiceEndpoint description as a DOM Document
     * 
     * @param endpoint
     *            the endpoint
     * @return a DOM representation of a ServiceEndpoint description or <code>null</code> if there is no such
     *         description
     * @throws EndpointDoesNotExistsException
     *             if the endpoint does not exist
     * @throws EndpointDirectoryException
     *             if an error occurs during endpoint description retrieval or if the endpoint does not exist
     */
    public Document getDescription(ServiceEndpoint endpoint) throws EndpointDirectoryException;

    public ServiceEndpoint resolveEndpointReference(DocumentFragment epr) throws EndpointDirectoryException;

}
